
def make_reads_annotation(trimm_type: str, splits: list, protein_domain: str = '') -> str:
    """
    Annotate reads based on trimm type - class, subclass, order, family
    """

    if trimm_type == 'class':
        return splits[0]
    elif trimm_type == 'subclass':
        return splits[1]
    elif trimm_type == 'order':
        if len(splits) > 2:
            return splits[2]
    elif trimm_type == 'family':
        if len(splits) > 3:  # longer then 3, then add order annotation to final annotation
            return splits[2] + '//' + splits[-1]
    elif trimm_type == 'protein domain':
        return protein_domain
    elif trimm_type == 'protein domain with family':
        return splits[-1] + ':' + protein_domain
    else:
        return splits[-1]  # otherwise return last part of annotation

def make_clusters_annotation(trimm_type: str, splits: list) -> str:
    """
    Annotate clusters based on trimm type - class, subclass, order, family
    """

    if len(splits) > 2:
        if trimm_type == 'repeat type':
            return splits[2]
        if splits[2] == 'mobile_element' and trimm_type == 'class' and len(splits) > 3 or len(splits) > 4:
            return make_reads_annotation(trimm_type, splits[3:])
        else:
            return 'not mobile element'
    else:
        return 'Unknown'


def trimm(annotation: str, trimm_type: str, data_type: str) -> str:
    """
    Trimms annotation based on trimm type and type of data
    """

    if data_type == 'reads':
        classification, protein_domain = annotation.split(':')
        splits = classification.split('/')
        output_text = make_reads_annotation(trimm_type, splits, protein_domain)
    elif data_type == 'clusters':
        splits = annotation.split('/')
        output_text = make_clusters_annotation(trimm_type, splits)
    else:
        output_text = annotation


    return output_text
