from anytree import Node, RenderTree
from anytree.render import AsciiStyle


def make_classification_tree(input_file, output_dir):
    """Makes classification tree from all classification file using anytree module """

    # Creates set with unique tree branches
    unique_branches = set()
    with open(input_file) as data:
        for line in data.readlines():
            columns = line.strip().split('\t')
            branch = ":".join(columns[1:])
            unique_branches.add(branch)

    tree_dict = dict()

    # Creates dict that represents tree structure
    for branch in unique_branches:
        act_depth = tree_dict
        for b in branch.split(':'):
            if b not in act_depth.keys():
                act_depth[b] = dict()
            act_depth = act_depth[b]

    # Creates visualisation with depth first traversal
    root = Node('Mobile elements')
    depth_first(tree_dict, 0, root)
    out_file = open(output_dir + 'classification.txt', mode='w')
    for pre, fill, node in RenderTree(root, style=AsciiStyle()):
        print(f"{pre}{node.name}", file=out_file)
    out_file.close()


def depth_first(tree: dict, spaces, node: Node):
    """Recursive function for depth first tree traversal"""
    if len(tree.keys()) == 0:
        return

    for k in tree.keys():
        n = Node(k, node)
        depth_first(tree[k], spaces + 1, n)
        
        