from tkinter import filedialog
from tkinter import *
from tkinter import ttk
from tkinter import messagebox
from tkinter import TclError, Scrollbar

import pandas as pd
import shutil


import stats_values
import classification_tree
import visualisations
import quantitative_stats
import fasta_functions
import os


class Gui:
    """Class that represents app GUI"""
    def __init__(self):
        # set root as main Tk and define basic properties
        self.root = Tk()
        self.root.title('RE analýza')
        self.root.geometry('330x720')
        self.root.grid_columnconfigure(0, weight=1)

        # dictionaries that will contains all frames and variables
        self.frames = dict()
        self.vars = dict()

        # Sets gui widgets
        self.set_frames()
        self.set_vars()
        self.set_text_labels()
        self.set_buttons()
        self.set_check_buttons()
        self.set_entries()

        # Sets progress bar
        self.progress = ttk.Progressbar(self.frames['progress'], orient=HORIZONTAL, length=305, mode='determinate')
        self.progress.grid(row=0, column=0, sticky='WE', padx=5, pady=5)

        # Runs main loop
        self.root.mainloop()

    def set_frames(self):
        """Sets GUI frames and put it to frames dict"""

        # frame for samples name
        self.frames['sample_names'] = self.set_labelframe(master=self.root,
                                                          frame_cnf={'text': 'Názvy vzorků', 'bd': 3, 'height': 50,
                                                                     'width': 500},
                                                          grid_cnf={'row': 0, 'column': 0, 'sticky': 'WE', 'padx': 5},
                                                          column_cnf={0: 3, 1: 1, 2: 3, 3: 1})

        # frame with input and output entries
        self.frames['io'] = self.set_labelframe(master=self.root,
                                                frame_cnf={'text': 'Vstup a výstup', 'bd': 3, 'height': 300, 'width': 500},
                                                grid_cnf={'row': 1, 'column': 0, 'sticky': 'WE', 'padx': 5})

        # frame with progress bar
        self.frames['progress'] = self.set_labelframe(master=self.root,
                                                      frame_cnf={'text': 'Průběh analýzy', 'bd': 3},
                                                      grid_cnf={'row': 4, 'column': 0, 'sticky': 'WE', 'padx': 5, 'ipadx': 15,
                                                                'columnspan': 3})

        # frame with run options
        self.frames['options'] = self.set_labelframe(master=self.root,
                                                     frame_cnf={'text': 'Možnosti analýzy', 'bd': 3},
                                                     grid_cnf={'row': 3, 'column': 0, 'sticky': 'WE', 'padx': 5, 'ipadx': 15},
                                                     column_cnf={0: 1})

        # frame with reads annotation options
        self.frames['reads_annotation'] = self.set_labelframe(master=self.frames['options'],
                                                              frame_cnf={'text': 'Vizualizace anotovaných čtení', 'bd': 3},
                                                              grid_cnf={'row': 0, 'column': 0, 'sticky': 'WE', 'padx': 5},
                                                              column_cnf={0: 2, 1: 2, 3: 1})

        # frame with clusters annotation options
        self.frames['clusters_annotation'] = self.set_labelframe(master=self.frames['options'],
                                                                 frame_cnf={'text': 'Vizualizace anotovaných klastrů', 'bd': 3},
                                                                 grid_cnf={'row': 1, 'column': 0, 'sticky': 'WE', 'padx': 5},
                                                                 column_cnf={0: 2, 1: 2, 3: 1})

        # frame with TAREAN annotation options
        self.frames['tarean_annotation'] = self.set_labelframe(master=self.frames['options'],
                                                               frame_cnf={'text': 'Vizualizace TAREAN anotací', 'bd': 3},
                                                               grid_cnf={'row': 2, 'column': 0, 'sticky': 'WE', 'padx': 5},
                                                               column_cnf={0: 2, 1: 2, 3: 1})
        # frame with B enriched clusters options
        self.frames['b_clusters'] = self.set_labelframe(master=self.frames['options'],
                                                        frame_cnf={'text': 'B obohacené klastry', 'bd': 3},
                                                        grid_cnf={'row': 3, 'column': 0, 'sticky': 'WE', 'padx': 5},
                                                        column_cnf={0: 2, 1: 2, 3: 1})

        # frame with classification tree options
        self.frames['classification_tree'] = self.set_labelframe(master=self.frames['options'],
                                                                 frame_cnf={'text': 'REXdb', 'bd': 3},
                                                                 grid_cnf={'row': 4, 'column': 0, 'sticky': 'WE', 'padx': 5,
                                                                 'ipadx': 15},
                                                                 column_cnf={0: 1, 1: 2})


    @staticmethod
    def set_labelframe(master, frame_cnf: dict, grid_cnf: dict, column_cnf: dict = None) -> LabelFrame:
        frame = LabelFrame(master, **frame_cnf)
        frame.grid(**grid_cnf)
        if column_cnf:
            for index, weight in column_cnf.items():
                frame.grid_columnconfigure(index=index, weight=weight)
        return frame

    def set_vars(self):
        # definition of vars used in app

        # variables for samples name
        self.vars['first_sample'] = StringVar(self.frames['io'], value="00")
        self.vars['second_sample'] = StringVar(self.frames['io'], value="BB")


        # variable with path to input folder
        self.vars['input'] = StringVar(self.frames['io'], value="/cesta/k/vstupnimu/adresari")

        # variable with path to output folder
        self.vars['output'] = StringVar(self.frames['io'], value="/cesta/k/vystupnimu/adresari")


        # variable with reads annotation size limit
        self.vars['reads_size_limit'] = DoubleVar(self.frames['reads_annotation'], value=0.001)

        # variable with clusters annotation size limit
        self.vars['clusters_size_limit'] = DoubleVar(self.frames['clusters_annotation'], value=0.001)

        # variable with tarean annotation size limit
        self.vars['tarean_size_limit'] = DoubleVar(self.frames['clusters_annotation'], value=0.001)

        # variable with limit for B reads enrichment in clusters
        self.vars['b_clusters_limit'] = DoubleVar(self.frames['b_clusters'], value=0.8)

        # variables for check buttons
        # reads annotation check buttons variables
        self.vars['reads_pp'] = BooleanVar(self.frames['reads_annotation'], value=1)

        self.vars['reads_bp'] = BooleanVar(self.frames['reads_annotation'], value=1)

        # clusters annotation check buttons variables
        self.vars['clusters_pp'] = BooleanVar(self.frames['clusters_annotation'], value=1)

        self.vars['clusters_bp'] = BooleanVar(self.frames['clusters_annotation'], value=1)

        # variables for tarean annotation frame check buttons
        self.vars['tarean_pp'] = BooleanVar(self.frames['clusters_annotation'], value=1)
        self.vars['tarean_bp'] = BooleanVar(self.frames['clusters_annotation'], value=1)

        # variable for BB enriched reads extraction checkbutton
        self.vars['b_extraction'] = BooleanVar(self.frames['b_clusters'], value=1)

        # variable for extraction of tandem repetition consensus sequences
        self.vars['tarean_sequences'] = BooleanVar(self.frames['options'], value=1)

        # variable for tree of TE classification inclusion
        self.vars['classification_tree'] = BooleanVar(self.frames['classification_tree'], value=1)

        # variable with path to rexdb file
        self.vars['rexdb'] = StringVar(self.frames['classification_tree'], value="/cesta/k/rexdb/souboru")

        # variable for visualization of clusters structure
        self.vars['cluster_structure'] = BooleanVar(self.frames['b_clusters'], value=1)

        # variable for quantitative values checkbutton
        self.vars['quantitative_data'] = BooleanVar(self.frames['options'], value=1)

    def set_entries(self):
        # definition of entries used in app

        # entry for path of input folder
        input_entry = Entry(self.frames['io'], width=30, textvariable=self.vars['input'], bd=2)
        input_entry.xview_moveto(1)
        input_entry.grid(column=2, row=0, sticky=E)

        # entry for path of output folder
        output_entry = Entry(self.frames['io'], width=30, textvariable=self.vars['output'], bd=2)
        output_entry.xview_moveto(1)
        output_entry.grid(column=2, row=1, sticky=E)

        # entries for sample names
        first_sample_entry = Entry(self.frames['sample_names'], width=9, textvariable=self.vars['first_sample'], bd=2)
        first_sample_entry.xview_moveto(1)
        first_sample_entry.grid(column=1, row=0, sticky=W)

        second_sample_entry = Entry(self.frames['sample_names'], width=9, textvariable=self.vars['second_sample'], bd=2)
        second_sample_entry.xview_moveto(1)
        second_sample_entry.grid(column=3, row=0, sticky=W)

        # entry for size limit of reads annotation
        Entry(self.frames['reads_annotation'], width=8,
              textvariable=self.vars['reads_size_limit'], bd=2).grid(column=2, row=2, sticky=W, )

        # entry for size limit of clusters annotation
        Entry(self.frames['clusters_annotation'], width=8,
              textvariable=self.vars['clusters_size_limit'], bd=2).grid(column=2, row=2, sticky=E)

        # entry for size limit of tarean annotation
        Entry(self.frames['tarean_annotation'], width=8,
              textvariable=self.vars['tarean_size_limit'], bd=2).grid(column=2, row=2, sticky=E)

        # entry for size limit of B reads for B enrichment in clusters
        Entry(self.frames['b_clusters'], width=8,
              textvariable=self.vars['b_clusters_limit'], bd=2).grid(column=2, row=2, sticky=E)

        # entry for path to rexdb file
        path_entry = Entry(self.frames['classification_tree'], width=43, textvariable=self.vars['rexdb'], bd=2)
        path_entry.xview_moveto(1)
        path_entry.grid(column=1, row=1, sticky=W, padx=1)

    def set_check_buttons(self):
        # setting of apps check buttons

        # check buttons for reads annotation frame
        Checkbutton(self.frames['reads_annotation'], variable=self.vars['reads_pp'],
                    text="Koláčový graf").grid(row=0, column=0, sticky='W')

        Checkbutton(self.frames['reads_annotation'], variable=self.vars['reads_bp'],
                    text="Sloupcový graf").grid(row=1, column=0, sticky='W')

        # check buttons for clusters annotation frame
        Checkbutton(self.frames['clusters_annotation'], variable=self.vars['clusters_pp'],
                    text="Koláčový graf").grid(row=0, column=0, sticky='W')

        Checkbutton(self.frames['clusters_annotation'], variable=self.vars['clusters_bp'],
                    text="Sloupcový graf").grid(row=1, column=0, sticky='W')

        Checkbutton(self.frames['tarean_annotation'], variable=self.vars['tarean_pp'],
                    text="Koláčový graf").grid(row=0, column=0, sticky='W')
        Checkbutton(self.frames['tarean_annotation'], variable=self.vars['tarean_bp'],
                    text="Sloupcový graf").grid(row=1, column=0, sticky='W')

        Checkbutton(self.frames['b_clusters'], variable=self.vars['b_extraction'],
                    text="Extrakce čtení").grid(row=1, column=0, sticky='W')

        Checkbutton(self.frames['b_clusters'], variable=self.vars['cluster_structure'],
                    text="Vizualizace").grid(row=0, column=0, sticky='W')

        Checkbutton(self.frames['options'], variable=self.vars['tarean_sequences'],
                    text="Extrakce TAREAN konsenzuálních sekvencí").grid(row=5, column=0, sticky='W')

        Checkbutton(self.frames['classification_tree'], variable=self.vars['classification_tree'],
                    text="Zahrnout strom klasifikace TE").grid(row=0, column=0, sticky='W', columnspan=2)

        Checkbutton(self.frames['options'], variable=self.vars['quantitative_data'],
                    text="Vytvořit soubor s kvantitativními údaji").grid(row=6, column=0, sticky='W')

    def set_text_labels(self):
        # setting of text labels
        # labels in io frame
        Label(self.frames['io'], text="Vstupní adresář:").grid(column=0, row=0, sticky=E)
        Label(self.frames['io'], text="Výstupní adresář:").grid(column=0, row=1, sticky=E)

        # labels for sample names
        Label(self.frames['sample_names'], text="1. vzorek:").grid(column=0, row=0, sticky=W)
        Label(self.frames['sample_names'], text="2. vzorek:").grid(column=2, row=0, sticky=W)


        # labels in reads annotation frame
        Label(self.frames['reads_annotation'], text="Velikostní limit:").grid(column=1, row=2, sticky=E)

        # labels in clusters annotation frame
        Label(self.frames['clusters_annotation'], text="Velikostní limit:").grid(column=1, row=2, sticky=E)

        # labels in tarean annotation frame
        Label(self.frames['tarean_annotation'], text="Velikostní limit:").grid(column=1, row=2, sticky=E, )

        # labels in b clusters  frame
        Label(self.frames['b_clusters'], text="Velikostní limit:").grid(column=1, row=2, sticky=E)

    def set_buttons(self):
        input_folder_button = Button(self.frames['io'], text="...",
                                     command=lambda: self.browse(self.vars['input']), bd=3, width=1)
        input_folder_button.grid(column=1, row=0, padx=2)

        output_folder_button = Button(self.frames['io'], text="...",
                                      command=lambda: self.browse(self.vars['output']), bd=3, width=1)
        output_folder_button.grid(column=1, row=1)

        Button(self.root, text="START", command=lambda: self.process_RE(), bd=3).grid(row=5, column=0,
                                                                                                columnspan=3, pady=5)
        rexdb_folder_button = Button(self.frames['classification_tree'], text="...",
                                      command=lambda: self.browse(self.vars['rexdb'], file=True), bd=3, width=1)
        rexdb_folder_button.grid(column=0, row=1, sticky=E, padx=3, pady=1)

    def launch(self):
        self.root.mainloop()
        print('baba')

    def reset_bar(self):
        self.progress['value'] = 0
        self.root.update_idletasks()

    def step(self):
        if self.progress['value'] == 100:
            print('gaga')
            self.progress['value'] = 10
        else:
            self.progress['value'] += 10
        self.root.update_idletasks()

    def check_parameters(self):
        try:
            for key in ['reads_size_limit', 'clusters_size_limit', 'tarean_size_limit', 'b_clusters_limit']:
                self.vars[key].get()
        except TclError:
            raise ValueError

    def process_RE(self):

        try:
            self.check_parameters()

            log = []
            log.append("Konfigurace gui:\n")
            log.extend(self.summary())
            self.reset_bar()
            # checking of the '/' symbol at the end of path
            self.check_backslash(log)
            self.step()
            print('step1')

            # required files destinations
            # cluster table
            cluster_table_dest = self.vars['input'].get() + 'CLUSTER_TABLE.csv'
            # comparative analysis counts
            comparative_analysis_dest = self.vars['input'].get() + 'COMPARATIVE_ANALYSIS_COUNTS.csv'
            # cluster table
            cluster_table = pd.read_csv(cluster_table_dest, '\t', header=0, na_values='""', skiprows=6, index_col=1)
            log.append("Úspěšně načten soubor CLUSTER_TABLE.csv\n")

            # comparative analysis counts csv
            comparative_analysis_counts = pd.read_csv(comparative_analysis_dest, '\t', header=0, skiprows=2, index_col=0,
                                                      nrows=cluster_table.shape[0])
            log.append("Úspěšně načten soubor COMPARATIVE_ANALYSIS_COUNTS.csv\n")

            self.step()
            print('step2')

            # checks that output directory, if not create it
            self.check_outdir(log)

            self.step()
            print('step3')

            # check if classification tree option is set, if so then run the function
            self.classification_tree(log)

            self.step()
            print('step4')

            # check if clusters or tarean functions are set, if so then run them
            self.clusters_visualisation(cluster_table, log)

            self.step()
            print('step6')

            # check if reads functions are set, if so, then run them
            self.reads_visualisation(cluster_table, log)

            self.step()
            print('step7')

            # check if cluster structure or b extraction functions are set, if so, then run them
            self.b_clusters(comparative_analysis_counts, log)

            self.step()
            print('step8')

            # check if tarean sequences function is set, if so, then run it
            self.tarean_sequences(log)

            self.step()
            print('step9')

            # check if quantitative data function is set, if so, run it
            self.quantitative_data(cluster_table_dest, cluster_table, comparative_analysis_counts, log)

            self.step()
            print('step5')

            # create log file
            self.create_log(log)



            self.step()
            print('step10')

        except FileNotFoundError:
            messagebox.showerror('Chyba', 'Špatně nadefinovaná cesta ke vstupnímu souboru, prosím zkuste to znovu')

        except ValueError:
            messagebox.showerror('Chyba', 'Špatná hodnota velikostního limitu, hodnota musí být desetinné číslo v intervalu (0, 1) ')

        except KeyError:
            messagebox.showerror('Chyba', 'Špatná hodnota názvů vzorků, názvy musí odpovídat názvům sloupců v souboru COMPARATIVE_ANALYSIS_COUNTS.csv ')


    def check_backslash(self, log):
        for key in ('input', 'output'):
            path = self.vars[key].get()
            if path[-1] != '/':
                self.vars[key].set(path + '/')
                log.append(f"Přidána koncovka '/' u {path}\n")

    def classification_tree(self, log):
        if self.vars['classification_tree'].get():
            classification_tree.make_classification_tree(self.vars['rexdb'].get(), self.vars['output'].get())
            log.append('Vytvořen klasifikační strom\n')

    def reads_visualisation(self, cluster_table, log):
        if self.vars['reads_pp'].get() or self.vars['reads_bp'].get():
            visualisations.annotated_reads_visualization(cluster_table.shape[0], self.vars, log)
            log.append('Dokončena vizualizace anotovaných readů\n')

    def clusters_visualisation(self, cluster_table, log):
        if self.vars['clusters_pp'].get() or self.vars['clusters_bp'].get() or self.vars['tarean_pp'].get() \
                or self.vars['tarean_bp'].get():
            visualisations.annotated_clusters_visualization(cluster_table, self.vars, log)
            log.append('Dokončena vizualizace anotovaných klastrů\n')

    def b_clusters(self, comparative_analysis_counts, log):
        if self.vars['cluster_structure'].get() or self.vars['b_extraction'].get():

            b_enriched_ids = visualisations.clusters_structure_visualization(comparative_analysis_counts,
                                                                             'Struktura klastrů', self.vars, log)

            log.append('Dokončena vizualizace struktury klastrů\n')

            if self.vars['b_extraction'].get():
                try:
                    os.makedirs(self.vars['output'].get() + 'b_enriched/')
                    log.append("Vytvořena složka b_enriched\n")
                except FileExistsError:
                    pass
                fasta_functions.b_extraction(b_enriched_ids.index, self.vars, log)

    def tarean_sequences(self, log):
        if self.vars['tarean_sequences'].get():
            tarean_output = self.vars['output'].get() + 'tarean_sequences/'
            try:
                os.mkdir(tarean_output)
                log.append("Vytvořena složka tarean_sequences")
            except FileExistsError:
                pass
            shutil.copy2(self.vars['input'].get() + 'TAREAN_consensus_rank_1.fasta',
                         tarean_output + 'putative_satellites_high_confidence.fasta')
            shutil.copy2(self.vars['input'].get() + 'TAREAN_consensus_rank_2.fasta',
                         tarean_output + 'putative_satellites_low_confidence.fasta')
            shutil.copy2(self.vars['input'].get() + 'TAREAN_consensus_rank_3.fasta',
                         tarean_output + 'putative_LTR_elements.fasta')
            shutil.copy2(self.vars['input'].get() + 'TAREAN_consensus_rank_4.fasta',
                         tarean_output + 'rDNA.fasta')

    def quantitative_data(self, cluster_table_dest, cluster_table, comparative_analysis_counts, log):
        if self.vars['quantitative_data'].get():
            numerical_stats = pd.read_csv(cluster_table_dest, sep='\t', nrows=5, names=['Hodnota'], index_col=0)
            numerical_stats.index = ['Počet čtení v klastrech', 'Počet klastrů', 'Počet superklastrů',
                                     'Počet singletů', 'Počet analyzovaných čtení']
            numerical_stats = numerical_stats.append(
                pd.DataFrame([cluster_table.shape[0]], index=['Počet anotovaných klastrů'], columns=['Hodnota']))

            next_stats = quantitative_stats.extract_quantitative_stats(comparative_analysis_counts, self.vars)

            numerical_stats = numerical_stats.append(next_stats)
            numerical_stats.to_csv(self.vars['output'].get() + 'quantitative_data.csv', sep='\t',
                                   index=True, header=False, encoding='utf-8-sig')
            log.append("Vytvořen soubor s kvantitativními údaji")

            stats_values.create_values_files(self.vars, comparative_analysis_counts, log)

    def create_log(self, log):
        log_file = open(self.vars['output'].get() + 'log.txt', mode='w')
        log_file.writelines(log)
        log_file.close()

    @staticmethod
    def browse(var: StringVar, file=False):
        if file:
            path = filedialog.askopenfile()
        else:
            path = filedialog.askdirectory()
        var.set(path)

    def check_outdir(self, log):
        try:
            os.mkdir(self.vars['output'].get())
            log.append("Vytvořena nová výstupní složka\n")
        except FileExistsError:
            log.append("Výstupní složka existuje\n")
            pass

    def summary(self):
        log = []
        for key in self.vars.keys():
            log.append(f"{key}:{self.vars[key].get()}\n")
        return log
