import io

import pandas
import pandas as pd
import numpy as np
import fasta_functions
import os


def row_names() -> list:
    return ["Počet BB obohacených klastrů",
                  "Minimální počet BB čtení tvořících BB obohacený klastr",
                  "Maximální počet BB čtení tvořících BB obohacený klastr",
                  "Průměrný počet BB čtení tvořících BB obohacený klastr",
                  "Minimální počet čtení tvořících BB obohacený klastr",
                  "Maximální počet čtení tvořících BB obohacený klastr",
                  "Průměrný počet čtení tvořících BB obohacený klastr",
                  "Minimální počet čtení tvořících klastr",
                  "Maximální počet čtení tvořících klastr",
                  "Průměrný počet čtení tvořících klastry",
                  "Minimální poměr BB čtení tvořících BB obohacený klastr",
                  "Maximální poměr BB čtení tvořících BB obohacený klastr",
                  "Průměrný poměr BB čtení tvořících BB obohacený klastr",
                  "Minimální poměr BB čtení tvořících klastr",
                  "Maximální poměr BB čtení tvořících klastr",
                  "Průměrný poměr BB čtení tvořících klastry"]

def comparative_data_stats(comparative_data: io. TextIOWrapper) -> tuple:
    
    comparative_data['all'] = comparative_data['BB'] + comparative_data['00']
    comparative_data['bb_ratio'] = comparative_data['BB'] / comparative_data['all']
    bb_enriched = comparative_data[comparative_data['bb_ratio'] > 0.8]

    stats = [len(bb_enriched),
             bb_enriched['BB'].min(), bb_enriched['BB'].max(), bb_enriched['BB'].mean(),
             bb_enriched['all'].min(), bb_enriched['all'].max(), bb_enriched['all'].mean(),
             comparative_data['all'].min(), comparative_data['all'].max(), comparative_data['all'].mean(),
             bb_enriched['bb_ratio'].min(), bb_enriched['bb_ratio'].max(), bb_enriched['bb_ratio'].mean(),
             comparative_data['bb_ratio'].min(), comparative_data['bb_ratio'].max(),
             comparative_data['bb_ratio'].mean()]

    return bb_enriched, stats

def bb_enriched_stats(bb_enriched: pd.DataFrame, variables: dict) -> tuple:
    contigs_lens = np.zeros(len(bb_enriched), dtype=np.int32)
    contigs_counts = np.zeros(len(bb_enriched), dtype=np.int32)
    for i, cl_id in enumerate(bb_enriched.index):
        contig_path = variables['output'].get() + f'b_enriched/{cl_id}/contigs.fasta'
        contigs_len, contigs_count = fasta_functions.seqs_process(contig_path)
        contigs_lens[i] = contigs_len
        contigs_counts[i] = contigs_count
    stats = [contigs_lens.min(), contigs_lens.max(), contigs_lens.mean(),
                  contigs_counts.min(), contigs_counts.max(), contigs_counts.mean()]
    stats_rows = ["Minimální délka kontigu", "Maximální délka kontigu", "Průměrná délka kontigu",
                       "Minimální počet kontigů", "Maximální počet kontigů", "Průměrný počet kontigů"]

    return stats, stats_rows


def tarean_data_stats(variables: dict) -> tuple:
    stats_rows = []
    stats = []
    for tarean_file in os.scandir(variables['output'].get() + 'tarean_sequences/'):
        row_name = " ".join(tarean_file.name[:-6].split('_'))

        stats_rows.extend([row_name + ' průměrná délka', row_name + ' počet '])
        seqs_len, seqs_count = fasta_functions.seqs_process(tarean_file.path)
        stats.extend([seqs_len, seqs_count])

    return stats, stats_rows


def extract_quantitative_stats(comparative_data: io.TextIOWrapper, variables: dict) -> pandas.DataFrame:
    stats_rows = row_names()

    bb_enriched, stats = comparative_data_stats(comparative_data)
    

    if variables['b_extraction'].get():
        bb_stats, bb_stats_rows = bb_enriched_stats(bb_enriched, variables)
        stats.extend(bb_stats)
        stats_rows.extend(bb_stats_rows)

    if variables['tarean_sequences']:
        tarean_stats, tarean_rows = tarean_data_stats(variables)
        stats.extend(tarean_stats)
        stats_rows.extend(tarean_rows)

    stats_df = pd.DataFrame(stats, index=stats_rows, columns=['Hodnota'])

    return stats_df
