import os
import pandas as pd
import numpy as np
import fasta_functions

def comparative_data_values(comparative_data: pd.DataFrame, output_dir: str, log: list) -> pd.DataFrame:

    comparative_data['all'] = comparative_data['BB'] + comparative_data['00']
    comparative_data['bb_ratio'] = comparative_data['BB'] / comparative_data['all']
    bb_enriched = comparative_data[comparative_data['bb_ratio'] > 0.8]
    bb_enriched.to_csv(output_dir + 'values/comparative_data.csv', sep='\t')
    return bb_enriched


def bb_enriched_values(bb_enriched: pd.DataFrame, variables: dict) -> tuple:
    contigs_lens = np.zeros(len(bb_enriched), dtype=np.int32)
    contigs_counts = np.zeros(len(bb_enriched), dtype=np.int32)
    for i, cl_id in enumerate(bb_enriched.index):
        contig_path = variables['output'].get() + f'b_enriched/{cl_id}/contigs.fasta'
        contigs_len, contigs_count = fasta_functions.seqs_process(contig_path)
        contigs_lens[i] = contigs_len
        contigs_counts[i] = contigs_count
    stats = [contigs_lens.min(), contigs_lens.max(), contigs_lens.mean(),
                  contigs_counts.min(), contigs_counts.max(), contigs_counts.mean()]
    stats_rows = ["Minimální délka kontigu", "Maximální délka kontigu", "Průměrná délka kontigu",
                       "Minimální počet kontigů", "Maximální počet kontigů", "Průměrný počet kontigů"]

    return stats, stats_rows


def contig_values(ids: list, output_dir: str) -> None:
    """
    Create csv file with values of contig lenght
    :param ids:
    :return:
    """
    row_ids = list()
    values = list()

    for id in ids:
        new_values = fasta_functions.fasta_seqs_len(output_dir + f'b_enriched/{id}/contigs.fasta')
        row_ids.extend([id] * len(new_values))
        values.extend(new_values)

    result = pd.DataFrame({'cluster_id': row_ids, 'contig length': values})
    result.to_csv(output_dir + 'values/contigs_length.csv')

def tarean_values(variables: dict, output_dir: str) -> None:
    ids = list()
    values = list()
    for tarean_file in os.scandir(output_dir + 'tarean_sequences/'):
        row_name = " ".join(tarean_file.name[:-6].split('_'))
        new_values = fasta_functions.fasta_seqs_len(tarean_file.path)
        values.extend(new_values)
        ids.extend([row_name] * len(new_values))

    result = pd.DataFrame({'cluster_id': ids, 'contig length': values})
    result.to_csv(output_dir + 'values/tarean_length.csv')



def create_values_files(variables: dict, comparative_data: pd.DataFrame, log: list) -> None:
    """
    Creates dir with csv files containing values of quantitative properties
    :param variables: dict with gui variables
    :param log: list with log sentcences
    :return: None, only create files
    """
    output_dir = variables['output'].get()
    try:
        os.mkdir(output_dir + 'values/')
        log.append("Vytvořena nová výstupní složka\n")
    except FileExistsError:
        log.append("Výstupní složka existuje\n")
        pass

    bb_enriched = comparative_data_values(comparative_data, output_dir, log)

    if variables['b_extraction'].get():
        contig_values(bb_enriched.index, output_dir)

    if variables['tarean_pp'].get() or variables['tarean_bp']:
        tarean_values(variables, output_dir)


