import shutil
import os

import pandas as pd


def fasta_search(file_dest, searched_prefix):
    prefix_len = len(searched_prefix)
    right_sequence = False
    lines = []
    with open(file_dest) as fasta_file:
        for line in fasta_file.readlines():
            if line[0] == '>':
                if line[1:prefix_len + 1] == searched_prefix:
                    right_sequence = True
                    lines.append(line)
                    continue
                else:
                    right_sequence = False
                    continue
            if right_sequence:
                lines.append(line)
    return lines


def b_extraction(ids, variables, log):
    output_dest = variables['output'].get() + 'b_enriched/'
    input_dir = variables['input'].get()

    for id in ids:
        id = str(id)
        cluster_dest = f"{input_dir}seqclust/clustering/clusters/dir_CL{(4 - len(id)) * '0' + id}/"
        cluster_out = output_dest + id + '/'
        try:
            os.mkdir(cluster_out)
            log.append(f'Vytvořena složka s klastrem {id}\n')
        except FileExistsError:
            pass
        try:
            shutil.copy2(cluster_dest + 'contigs.fasta', cluster_out)
            log.append(f'Přemístěn soubor contigs.fasta do složky {id}\n')
        except FileNotFoundError:
            log.append(f'Soubor contigs.fasta nenalezen v klastrové složce, hledání v hlavním adresáři\n')
            lines = fasta_search(input_dir + 'contigs.fasta', 'CL' + id + 'C')
            with open(cluster_out + 'contigs.fasta', mode='w') as out_file:
                out_file.writelines(lines)

        reads_out = open(cluster_out + 'reads.fasta', mode='w')
        b_read = False
        for line in open(cluster_dest + 'reads.fasta'):
            if line[0] == '>':
                if line[1:3] == 'BB':
                    b_read = True
                    reads_out.write(line)
                else:
                    b_read = False
                continue
            if b_read:
                reads_out.write(line)
        log.append(f'Vytvořen soubor reads.fasta do složky {id}\n')

        reads_out.close()


def seqs_process(contig_path):
    contig_count = 0
    contig_len = 0
    with open(contig_path) as contig:
        for line in contig.readlines():
            if line[0] == '>':
                contig_count += 1
            else:
                contig_len += len(line.strip())

    return contig_len, contig_count

def fasta_seqs_len(fasta_path: str) -> list:
    """
    Extract lengths of seqs of fasta  file
    :param cluster_path:
    :return:
    """
    contig_lens = []

    with open(fasta_path) as fasta:
        contig_len = 0
        for line in fasta.readlines():
            if line[0] == '>':
                if contig_len:
                    contig_lens.append(contig_len)
                contig_len = 0
            else:
                contig_len += len(line.strip())
        if contig_len:
            contig_lens.append(contig_len)

    return contig_lens

